package com.barrelsofdata.examples.beam.streaming.sensor.transform;

import com.barrelsofdata.examples.beam.streaming.sensor.config.Steps;
import com.barrelsofdata.examples.beam.streaming.sensor.function.Average;
import com.barrelsofdata.examples.beam.streaming.sensor.function.ExtractKeys;
import com.barrelsofdata.examples.beam.streaming.sensor.model.ComputedEvent;
import com.barrelsofdata.examples.beam.streaming.sensor.model.RawEvent;
import org.apache.beam.sdk.transforms.Combine;
import org.apache.beam.sdk.transforms.PTransform;
import org.apache.beam.sdk.transforms.Values;
import org.apache.beam.sdk.transforms.WithKeys;
import org.apache.beam.sdk.transforms.windowing.AfterPane;
import org.apache.beam.sdk.transforms.windowing.AfterWatermark;
import org.apache.beam.sdk.transforms.windowing.SlidingWindows;
import org.apache.beam.sdk.transforms.windowing.Window;
import org.apache.beam.sdk.values.PCollection;
import org.joda.time.Duration;

/**
 * PTransform that applies sliding windows and computes average from a group of @{@link RawEvent}
 * id from the event is used for grouping the data
 * Emits a {@link ComputedEvent} for every window group
 * id and timestamp for the emitted are fetched from the last event in the group
 */
public class ComputeAverage extends PTransform<PCollection<RawEvent>, PCollection<ComputedEvent>> {

    private final Long windowDurationSeconds;
    private final Long windowFrequencySeconds;
    private final Long allowedLatenessSeconds;
    private final Window<RawEvent> window;

    public ComputeAverage(Long windowDurationSeconds, Long windowFrequencySeconds, Long allowedLatenessSeconds) {
        this.windowDurationSeconds = windowDurationSeconds;
        this.windowFrequencySeconds = windowFrequencySeconds;
        this.allowedLatenessSeconds = allowedLatenessSeconds;
        this.window = configureWindow();
    }

    @Override
    public PCollection<ComputedEvent> expand(PCollection<RawEvent> input) {

        PCollection<RawEvent> windowed = input.apply(Steps.ADD_WINDOWS.getStep(), window);
        return windowed
                .apply(Steps.EXTRACT_KEY.getStep(), WithKeys.of(new ExtractKeys()))
                .apply(Steps.COMPUTE_AVERAGE.getStep(), Combine.perKey(new Average()))
                .apply(Steps.REMOVE_KEY.getStep(), Values.create());
    }

    private Window<RawEvent> configureWindow() {
        return Window.<RawEvent>into(SlidingWindows
                        .of(Duration.standardSeconds(windowDurationSeconds))
                        .every(Duration.standardSeconds(windowFrequencySeconds)))
                .withAllowedLateness(Duration.standardSeconds(allowedLatenessSeconds))
                .accumulatingFiredPanes()
                .triggering(AfterWatermark.pastEndOfWindow()
                        .withLateFirings(AfterPane.elementCountAtLeast(1)));
    }
}
