package com.barrelsofdata.examples.beam.streaming.sensor.function;

import com.barrelsofdata.examples.beam.streaming.sensor.model.ComputedEvent;
import com.barrelsofdata.examples.beam.streaming.sensor.model.RawEvent;
import com.barrelsofdata.examples.beam.streaming.sensor.testutils.RawEventsGenerator;
import org.joda.time.Instant;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.CsvSource;

import java.util.ArrayList;
import java.util.stream.Collectors;

import static org.junit.jupiter.api.Assertions.*;

class AverageTest {

    @ParameterizedTest
    @CsvSource(value = {
            "testId1|2022-06-01T10:00:00.000Z|10|1.0|0.5|20|5.75",
            "testId1|2022-06-01T10:00:00.000Z|10|1.0|0.0|20|1.0"
    }, delimiter = '|')
    void testThatComputedAveragesAreCorrect(String id, String startTimestamp, Long timestampStepSeconds, Double startValue, Double valueSteps, int count, Double expectedAverage) {
        ArrayList<RawEvent> rawEvents = RawEventsGenerator.generate(id, startTimestamp, timestampStepSeconds, startValue, valueSteps, count);
        int half = rawEvents.size() / 2;
        ArrayList<ArrayList<RawEvent>> workerSplits = new ArrayList<>(rawEvents.stream()
                .collect(Collectors.partitioningBy(
                        s -> rawEvents.indexOf(s) > half))
                .values().stream().map(ArrayList::new).toList());

        Average average = new Average();
        ArrayList<RawEvent> merged = average.mergeAccumulators(workerSplits);
        ComputedEvent computed = average.extractOutput(merged);

        assertEquals(id, computed.id(), "Id in ComputedEvent is wrong");
        assertEquals(new Instant(startTimestamp).getMillis() + (timestampStepSeconds * 1000 * --count), computed.ts(), "Timestamp of ComputedEvent is wrong");
        assertEquals(expectedAverage, computed.value(), "Average computed is incorrect");
    }

    @ParameterizedTest
    @CsvSource(value = {
            "testId1|2022-06-01T10:00:00.000Z|10|1.0|0.5|20",
            "testId1|2022-06-01T10:00:00.000Z|10|1.0|0.0|25"
    }, delimiter = '|')
    void testThatAccumulatorMergeIsWorking(String id, String startTimestamp, Long timestampStepSeconds, Double startValue, Double valueSteps, int count) {
        ArrayList<RawEvent> rawEvents = RawEventsGenerator.generate(id, startTimestamp, timestampStepSeconds, startValue, valueSteps, count);
        int half = rawEvents.size() / 2;
        ArrayList<ArrayList<RawEvent>> workerSplits = new ArrayList<>(rawEvents.stream()
                .collect(Collectors.partitioningBy(
                        s -> rawEvents.indexOf(s) > half))
                .values().stream().map(ArrayList::new).toList());

        Average average = new Average();
        ArrayList<RawEvent> merged = average.mergeAccumulators(workerSplits);
        assertEquals(count, merged.size(), "Merged accumulator size does not match with input list");
        assertTrue(rawEvents.containsAll(merged) && merged.containsAll(rawEvents), "Merged accumulator missing elements from input list");
    }

    @ParameterizedTest
    @CsvSource(value = {
            "testId1|2022-06-01T10:00:00.000Z|10|1.0|0.5|0",
            "testId1|2022-06-01T10:00:00.000Z|10|1.0|0.0|25"
    }, delimiter = '|')
    void testThatElementsAreAddedToAccumulators(String id, String startTimestamp, Long timestampStepSeconds, Double startValue, Double valueSteps, int count) {
        ArrayList<RawEvent> rawEvents = RawEventsGenerator.generate(id, startTimestamp, timestampStepSeconds, startValue, valueSteps, count);
        RawEvent eventToAdd = RawEventsGenerator.generateOne();

        Average average = new Average();
        ArrayList<RawEvent> merged = average.addInput(rawEvents, eventToAdd);
        assertEquals(count + 1, merged.size(), "Element was not added to accumulator");
        assertTrue(merged.contains(eventToAdd), "Element was not found in the accumulator");
    }

}