package com.barrelsofdata.examples.beam.streaming.sensor.testutils.kafka;

import org.apache.kafka.clients.producer.KafkaProducer;
import org.apache.kafka.clients.producer.ProducerConfig;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.apache.kafka.common.serialization.StringSerializer;
import org.springframework.kafka.test.EmbeddedKafkaBroker;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.util.Properties;

public class EmbeddedKafka {
    private static final int NUMBER_OF_BROKERS = 1;
    private final EmbeddedKafkaBroker embeddedKafkaBroker;
    private final Producer kafkaProducer;

    public static EmbeddedKafka withDefaults() {
        return new EmbeddedKafka(NUMBER_OF_BROKERS);
    }

    public EmbeddedKafka(int numBrokers) {
        validate(numBrokers);
        embeddedKafkaBroker = new EmbeddedKafkaBroker(numBrokers);
        embeddedKafkaBroker.brokerProperty("log.dir", "build/embedded-kafka/logs");
        kafkaProducer = new Producer(embeddedKafkaBroker.getBrokersAsString());
    }

    public void start() {
        embeddedKafkaBroker.afterPropertiesSet();
    }

    public void addTopics(String... topics) {
        embeddedKafkaBroker.addTopics(topics);
    }

    public void send(String topic, String key, String message) {
        kafkaProducer.send(topic, key, message);
    }

    public void sendFile(String topic, String filepath, String delimiter) throws FileNotFoundException {
        BufferedReader br = new BufferedReader(new FileReader(filepath));
        br.lines().forEach(line -> {
            String[] pairs = line.split(delimiter);
            assert pairs.length > 1;
            send(topic, pairs[0], pairs[1]);
        });
    }

    public void stop() {
        embeddedKafkaBroker.destroy();
    }

    public String brokers() {
        return embeddedKafkaBroker.getBrokersAsString();
    }

    private void validate(int numBrokers) {
        if(numBrokers < 1)
            throw new RuntimeException("Number of brokers should be atleast 1");
    }

    private static class Producer {

        private final KafkaProducer<String, String> kafkaProducer;

        public Producer(String bootstrapServers) {
            Properties properties = new Properties();
            properties.setProperty(ProducerConfig.BOOTSTRAP_SERVERS_CONFIG, bootstrapServers);
            properties.setProperty(ProducerConfig.KEY_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());
            properties.setProperty(ProducerConfig.VALUE_SERIALIZER_CLASS_CONFIG, StringSerializer.class.getName());
            kafkaProducer = new KafkaProducer<>(properties);
        }

        public void send(String topic, String key, String value) {
            var record = new ProducerRecord<>(topic, key, value);
            kafkaProducer.send(record);
            kafkaProducer.flush();
        }
    }
}
